package Grenouille::Preferences;

# This software may be used and distributed according to the terms
# of the GNU General Public License, incorporated herein by reference.

use strict;
use Exporter qw();
use Grenouille::Vars qw($MYRC);
use vars qw(@ISA @EXPORT_OK %preferences);
@ISA=qw(Exporter);
@EXPORT_OK=qw(load_preferences store_preferences 
              ask_preferences get_preferences
              %preferences);

%preferences = ();

sub load_preferences() {
    Grenouille::System::swap_privileges();
    eval {
	if (open(CONFIG, "<$MYRC")) {
	    while (<CONFIG>) {
		chomp;
		s/#.*$//;
		$preferences{$1} = $2 if /^([^=]*)=(.*)$/;
	    }
	    close(CONFIG);
	}
    };
    Grenouille::System::swap_privileges();
    die if $@;
}

sub store_preferences() {
    my($oldumask) = umask 0177;
    Grenouille::System::swap_privileges();
    eval {
	if(open(CONFIG, ">$MYRC")) {
	    map { print CONFIG "$_=$preferences{$_}\n"; } sort keys %preferences;
	    close(CONFIG);
	    chown $<, -1, $MYRC;
	    chmod 0600, $MYRC;
	} else {
	    display_and_log('WARNING', "Ecriture du fichier $MYRC impossible: vrifier les permissions");
	}
    };
    Grenouille::System::swap_privileges();
    umask $oldumask;
    die if $@;
}

sub _ask($$$) {
    my($msg, $type, $default) = @_;
    my($answer);
    print $msg;
    if(defined $default) {
	print ' (';
	print $default ? 'O/n' : 'o/N' if $type eq 'boolean';
	print $default if $type eq 'string';
	print '******' if $type eq 'hidden_string';
	print ')';
    }
    print ': ';
    chomp($answer = <STDIN>);
    return $default unless $answer;
    return $answer !~ /n/ ? 1 : 0 if $type eq 'boolean';
    return $answer if ($type eq 'string') or ($type eq 'hidden_string');
    die 'unknown answer type';
}

sub ask_preferences() {
    die 'Ce programme doit tre lanc interactivement pour tre configur' unless -t STDIN and -t STDOUT;

    load_preferences();
    
    print "\nEntrez les valeurs qui vous ont servi  vous enregistrer sur grenouille.com :\n\n";
    $preferences{'user'} = _ask('Pseudo', 'string', $preferences{'user'});

    $preferences{'pass'} = _ask('Mot de passe', 'hidden_string', $preferences{'pass'});

    $preferences{'email'} = _ask('Adresse email', 'string', $preferences{'email'});

    print "\nplgrenouille utilise le protocole SNMP pour observer le rseau durant ses\n" .
          "test. Le p?riph?rique d?sign? *doit* voir tout le traffic provenant ou ?\n" .
          "destination de votre connexion. Les arborescences 'interfaces' et 'ip' doivent\n" .
          "tre accessibles en lecture.\n";

    $preferences{'snmp_host'} = _ask('P?riph?rique SNMP', 'string', defined $preferences{'snmp_host'} ? $preferences{'snmp_host'} : 'localhost');

    $preferences{'snmp_community'} = _ask('Communaut? SNMP', 'string', defined $preferences{'snmp_community'} ? $preferences{'snmp_community'} : 'public');

    print "\nVeuillez spcifier vos prfrences :\n";

    $preferences{'test_upload'} = _ask(
       "\nATTENTION:\nCertains abonnements (en gnral le cable) imposent des restrictions en\n" .
       "upload; La mesure de l'upload peut consommer une portion non ngligeable\n" .
       "de votre forfait.\nMesurer l'upload", 
       'boolean', defined $preferences{'test_upload'} ? ($preferences{'test_upload'} ne '0') : 1);

    $preferences{'passive_ftp'} = _ask(
       "\nPour des raisons de compatibilit et de scurit, le mode FTP \"passif\"\n" .
       "est aujourd'hui le mode FTP de prdilection.\n" .
       "(lire http://cr.yp.to/ftp/security.html pour plus d'informations)\n" .
       'Voulez-vous utiliser FTP en mode passif', 
       'boolean', defined $preferences{'passive_ftp'} ? ($preferences{'passive_ftp'} ne '0') : 1);

    $preferences{'email_notification'} = _ask(
      "\nSi vous comptez lancer plgrenouille automatiquement au boot, il n'aura\n" .
      "probablement aucune chance de pouvoir vous avertir lors de la sortie de\n" .
      "nouvelles versions.\n" .
      'Voulez-vous tre averti des nouvelles versions par email', 
      'boolean', defined $preferences{'email_notification'} ? ($preferences{'email_notification'} ne 0) : 1);
    
    print "\nLa configuration sera sauvegarde dans $MYRC\n";
    store_preferences();
}

sub get_preferences() {
    load_preferences();
    
    die "plgrenouille n'a pas t totalement paramtr, lancez plgrenouille --config ou modifiez $MYRC" 
	if not defined $preferences{'user'} or 
	   not defined $preferences{'pass'} or
	   not defined $preferences{'email'} or 
           not defined $preferences{'snmp_host'} or
           not defined $preferences{'snmp_community'} or
	   not defined $preferences{'test_upload'} or
	   not defined $preferences{'email_notification'} or
	   not defined $preferences{'passive_ftp'};
}

1;
